@@ -31,6 +31,7 @@
 
 #include <JavaScriptCore/JSObjectRef.h>
 #include <JavaScriptCore/JSRetainPtr.h>
+#include <map>
 #include <set>
 #include <string>
 #include <vector>
@@ -45,6 +46,8 @@ public:
     void makeWindowObject(JSContextRef context, JSObjectRef windowObject, JSValueRef* exception);
 
     void addDisallowedURL(JSStringRef url);
+    void addURLToRedirect(std::string origin, std::string destination);
+    const std::string& redirectionDestinationForURL(std::string);
     void clearAllApplicationCaches();
     void clearAllDatabases();
     void clearBackForwardList();
@@ -56,6 +59,7 @@ public:
     void disableImageLoading();
     void dispatchPendingLoadRequests();
     void display();
+    void displayInvalidatedRegion();
     void execCommand(JSStringRef name, JSStringRef value);
     bool findString(JSContextRef, JSStringRef, JSObjectRef optionsArray);
     bool isCommandEnabled(JSStringRef name);
@@ -64,6 +68,7 @@ public:
     JSValueRef nodesFromRect(JSContextRef, JSValueRef, int x, int y, unsigned top, unsigned right, unsigned bottom, unsigned left, bool ignoreClipping);
     void notifyDone();
     int numberOfPages(float pageWidthInPixels, float pageHeightInPixels);
+    int numberOfPendingGeolocationPermissionRequests();
     void overridePreference(JSStringRef key, JSStringRef value);
     int pageNumberForElementById(JSStringRef id, float pageWidthInPixels, float pageHeightInPixels);
     JSRetainPtr<JSStringRef> pageProperty(const char* propertyName, int pageNumber) const;
@@ -74,6 +79,7 @@ public:
     void queueForwardNavigation(int howFarForward);
     void queueLoad(JSStringRef url, JSStringRef target);
     void queueLoadHTMLString(JSStringRef content, JSStringRef baseURL);
+    void queueLoadAlternateHTMLString(JSStringRef content, JSStringRef baseURL, JSStringRef unreachableURL);
     void queueLoadingScript(JSStringRef script);
     void queueNonLoadingScript(JSStringRef script);
     void queueReload();
@@ -84,6 +90,7 @@ public:
     void setAppCacheMaximumSize(unsigned long long quota);
     void setApplicationCacheOriginQuota(unsigned long long quota);
     void setAuthorAndUserStylesEnabled(bool);
+    void setAutofilled(JSContextRef, JSValueRef nodeObject, bool autofilled);
     void setCacheModel(int);
     void setCustomPolicyDelegate(bool setDelegate, bool permissive);
     void setDatabaseQuota(unsigned long long quota);
@@ -106,6 +113,7 @@ public:
     void setUseDashboardCompatibilityMode(bool flag);
     void setUserStyleSheetEnabled(bool flag);
     void setUserStyleSheetLocation(JSStringRef path);
+    void setValueForUser(JSContextRef, JSValueRef nodeObject, JSStringRef value);
     void setViewModeMediaFeature(JSStringRef mode);
     void setXSSAuditorEnabled(bool flag);
     void setFrameFlatteningEnabled(bool enable);
@@ -286,8 +294,8 @@ public:
 
     bool hasSpellingMarker(int from, int length);
 
-    void dumpConfigurationForViewport(int availableWidth, int availableHeight);
-    
+    void dumpConfigurationForViewport(int deviceDPI, int deviceWidth, int deviceHeight, int availableWidth, int availableHeight);
+
     static void setSerializeHTTPLoads(bool serialize);
 
     // The following API test functions should probably be moved to platform-specific 
@@ -302,9 +310,20 @@ public:
 
     JSRetainPtr<JSStringRef> markerTextForListItem(JSContextRef context, JSValueRef nodeObject) const;
 
+    JSValueRef originsWithLocalStorage(JSContextRef);
+    void deleteAllLocalStorage();
+    void deleteLocalStorageForOrigin(JSStringRef originIdentifier);
+    void observeStorageTrackerNotifications(unsigned number);
+    void syncLocalStorage();
+
+    void setShouldPaintBrokenImage(bool);
+    bool shouldPaintBrokenImage() const { return m_shouldPaintBrokenImage; }
+
     static const unsigned maxViewWidth;
     static const unsigned maxViewHeight;
 
+    void setMinimumTimerInterval(double);
+
 private:
     LayoutTestController(const std::string& testPathOrURL, const std::string& expectedPixelHash);
 
@@ -351,6 +370,7 @@ private:
     bool m_handlesAuthenticationChallenges;
     bool m_isPrinting;
     bool m_deferMainResourceDataLoad;
+    bool m_shouldPaintBrokenImage;
 
     std::string m_authenticationUsername;
     std::string m_authenticationPassword; 
@@ -361,6 +381,8 @@ private:
     
     // origins which have been granted desktop notification access
     std::vector<JSStringRef> m_desktopNotificationAllowedOrigins;
+
+    std::map<std::string, std::string> m_URLsToRedirect;
     
     static JSClassRef getJSClass();
     static JSStaticValue* staticValues();
